@@ -1,10 +1,10 @@
-import numpy as np
+from py_wake import np
 from numpy import newaxis as na
-from py_wake.deficit_models import DeficitModel
 from py_wake.deficit_models import BlockageDeficitModel
-from py_wake.ground_models.ground_models import NoGround
 from py_wake.utils.gradients import hypot
 from py_wake.deficit_models.utils import a0
+from py_wake.wind_turbines._wind_turbines import WindTurbine
+from py_wake.wind_turbines.power_ct_functions import PowerCtFunctions
 
 
 class Rathmann(BlockageDeficitModel):
@@ -24,12 +24,10 @@ class Rathmann(BlockageDeficitModel):
         Journal of Physics: Conference Series 1934 (2021) 012023
     """
 
-    args4deficit = ['WS_ilk', 'D_src_il', 'dw_ijlk', 'cw_ijlk', 'ct_ilk']
-
-    def __init__(self, sct=1.0, limiter=1e-10, exclude_wake=True, superpositionModel=None, groundModel=NoGround(),
-                 upstream_only=False):
-        DeficitModel.__init__(self, groundModel=groundModel)
-        BlockageDeficitModel.__init__(self, upstream_only=upstream_only, superpositionModel=superpositionModel)
+    def __init__(self, sct=1.0, limiter=1e-10, exclude_wake=True, superpositionModel=None,
+                 rotorAvgModel=None, groundModel=None, upstream_only=False):
+        BlockageDeficitModel.__init__(self, upstream_only=upstream_only, superpositionModel=superpositionModel,
+                                      rotorAvgModel=rotorAvgModel, groundModel=groundModel)
         # limiter to avoid singularities
         self.limiter = limiter
         # coefficient for scaling the effective forcing
@@ -111,12 +109,10 @@ class RathmannScaled(Rathmann):
         Journal of Physics: Conference Series 1934 (2021) 012023
     """
 
-    args4deficit = ['WS_ilk', 'D_src_il', 'dw_ijlk', 'cw_ijlk', 'ct_ilk']
-
-    def __init__(self, sct=1.0, limiter=1e-10, exclude_wake=True, superpositionModel=None, groundModel=NoGround(),
-                 upstream_only=False):
-        DeficitModel.__init__(self, groundModel=groundModel)
-        BlockageDeficitModel.__init__(self, upstream_only=upstream_only, superpositionModel=superpositionModel)
+    def __init__(self, sct=1.0, limiter=1e-10, exclude_wake=True, superpositionModel=None,
+                 rotorAvgModel=None, groundModel=None, upstream_only=False):
+        BlockageDeficitModel.__init__(self, upstream_only=upstream_only, superpositionModel=superpositionModel,
+                                      rotorAvgModel=rotorAvgModel, groundModel=groundModel)
         # coefficients for BEM approximation by Madsen (1997)
         self.a0p = np.array([0.2460, 0.0586, 0.0883])
         # limiter to avoid singularities
@@ -153,11 +149,9 @@ def main():
         from py_wake.examples.data.iea37._iea37 import IEA37Site
         from py_wake.examples.data.iea37._iea37 import IEA37_WindTurbines
         from py_wake.site._site import UniformSite
-        from py_wake.wind_turbines import OneTypeWindTurbines
         from py_wake.superposition_models import LinearSum
         from py_wake.wind_farm_models import All2AllIterative
         from py_wake.deficit_models.no_wake import NoWakeDeficit
-        from py_wake.rotor_avg_models import RotorCenter
         from py_wake.deficit_models.vortexcylinder import VortexCylinder
         from py_wake.deficit_models.vortexdipole import VortexDipole
         import matplotlib.pyplot as plt
@@ -237,10 +231,11 @@ def main():
         plt.title('Rathmann model, AEP: %.3f GWh' % aep)
         plt.show()
 
-        class epfl_model_wt(OneTypeWindTurbines):
+        class epfl_model_wt(WindTurbine):
             def __init__(self):
-                OneTypeWindTurbines.__init__(self, 'NREL 5MW', diameter=2, hub_height=1,
-                                             ct_func=self._ct, power_func=self._power, power_unit='W')
+                WindTurbine.__init__(self, 'NREL 5MW', diameter=2, hub_height=1,
+                                     powerCtFunction=PowerCtFunctions(power_function=self._power, power_unit='w',
+                                                                      ct_function=self._ct))
 
             def _ct(self, u):
                 ct = 0.798
@@ -262,7 +257,7 @@ def main():
             def __init__(self):
                 p_wd = [1]
                 ws = [1]
-                ti = [0.06]
+                ti = 0.06
                 UniformSite.__init__(self, p_wd=p_wd, ti=ti, ws=ws)
                 self.initial_position = np.array([wt_x, wt_y]).T
 
@@ -281,7 +276,7 @@ def main():
                 print(nam, blockage_model)
                 # for dum, rotor_avg_model in rotor_avg_models:
                 wm = All2AllIterative(site, wt, wake_deficitModel=NoWakeDeficit(), superpositionModel=LinearSum(),
-                                      blockage_deficitModel=blockage_model, rotorAvgModel=RotorCenter())
+                                      blockage_deficitModel=blockage_model, rotorAvgModel=None)
                 res[nam] = wm(wt_x, wt_y, wd=[180., 195., 210., 225.], ws=[1.])
                 tic = time.perf_counter()
                 flow_map = res[nam].flow_map(grid=grid, ws=[1.], wd=225.)
@@ -301,7 +296,7 @@ def main():
         plt.figure()
         lines = ['.-', 'o--', '^', '-.', '.-', '.-']
         theta = [0, 15, 30, 45]
-        viridis = plt.cm.viridis(np.linspace(0, 0.9, 5))
+        viridis = plt.cm.viridis(np.linspace(0, 0.9, 5))  # @UndefinedVariable
         jj = 0
         tno = np.arange(1, 6, 1)
         for i in range(4):
