@@ -1,6 +1,6 @@
 from py_wake.site.wasp_grid_site import WaspGridSite, load_wasp_grd
 from py_wake.examples.data.ParqueFicticio import ParqueFicticio_path
-import numpy as np
+from py_wake import np
 from py_wake.site.distance import TerrainFollowingDistance
 
 """
