@@ -1,4 +1,4 @@
-import numpy as np
+from py_wake import np
 from py_wake.site._site import UniformWeibullSite
 from py_wake.wind_turbines import WindTurbine
 from py_wake.wind_turbines.power_ct_functions import PowerCtTabular
