@@ -1,8 +1,9 @@
-import numpy as np
+from py_wake import np
 import xarray as xr
 from numpy import newaxis as na
 from scipy.interpolate import InterpolatedUnivariateSpline
 import matplotlib.pyplot as plt
+from py_wake.utils.xarray_utils import ilk2da
 
 
 class FlowBox(xr.Dataset):
@@ -13,23 +14,30 @@ class FlowBox(xr.Dataset):
         self.windFarmModel = self.simulationResult.windFarmModel
         lw_j = localWind_j
         wd, ws = lw_j.wd, lw_j.ws
+        time = 'time' in simulationResult
 
         if X is None and Y is None and H is None:
             coords = localWind_j.coords
             X = localWind_j.i
         else:
-            coords = {'x': X[0, :, 0], 'y': Y[:, 0, 0], 'h': H[0, 0, :], 'wd': wd, 'ws': ws}
+            coords = {'x': X[0, :, 0], 'y': Y[:, 0, 0], 'h': H[0, 0, :]}
+            if time:
+                coords['time'] = lw_j.coords['time']
+        coords.update({k: (dep, v, {'Description': d}) for k, dep, v, d in [
+                          ('wd', ('wd', 'time')[time], wd, 'Ambient reference wind direction [deg]'),
+                          ('ws', ('ws', 'time')[time], ws, 'Ambient reference wind speed [m/s]')]})
 
         def get_da(arr_jlk):
             if len(X.shape) == 1:
-                return xr.DataArray(arr_jlk.reshape(X.shape + (len(wd), len(ws))), coords, dims=['i', 'wd', 'ws'])
+                return ilk2da(arr_jlk, coords)
             else:
-                return xr.DataArray(arr_jlk.reshape(X.shape + (len(wd), len(ws))),
-                                    coords, dims=['y', 'x', 'h', 'wd', 'ws'])
+                shape = [X.shape + (len(wd), len(ws)), X.shape + (len(wd), )][time]
+                dims = ['y', 'x', 'h'] + [['wd', 'ws'], ['time']][time]
+                return xr.DataArray(arr_jlk.reshape(shape), coords, dims)
         JLK = WS_eff_jlk.shape
-        xr.Dataset.__init__(self, data_vars={k: get_da(v) for k, v in [
+        xr.Dataset.__init__(self, data_vars={k: get_da(np.broadcast_to(v, JLK)) for k, v in [
             ('WS_eff', WS_eff_jlk), ('TI_eff', TI_eff_jlk),
-            ('WD', lw_j.WD.ilk(JLK)), ('WS', lw_j.WS.ilk(JLK)), ('TI', lw_j.TI.ilk(JLK)), ('P', lw_j.P.ilk(JLK))]})
+            ('WD', lw_j.WD_ilk), ('WS', lw_j.WS_ilk), ('TI', lw_j.TI_ilk), ('P', lw_j.P_ilk)]})
 
 
 class FlowMap(FlowBox):
@@ -43,15 +51,15 @@ class FlowMap(FlowBox):
         if plane[0] == 'XY':
             X = X[:, :, na]
             Y = Y[:, :, na]
-            H = np.reshape(localWind_j.h.data, X.shape)
+            H = np.reshape(localWind_j.h, X.shape)
         elif plane[0] == 'YZ':
             H = Y.T[:, na, :]
             Y = X.T[:, na, :]
-            X = np.reshape(localWind_j.x.data, Y.shape)
+            X = np.reshape(localWind_j.x, Y.shape)
         elif plane[0] == 'XZ':
             H = Y.T[:, na, :]
             X = X.T[na, :, :]
-            Y = np.reshape(localWind_j.y.data, X.shape)
+            Y = np.reshape(localWind_j.y, X.shape)
         elif plane[0] == 'xyz':
             X = None
             Y = None
@@ -74,6 +82,9 @@ class FlowMap(FlowBox):
                 self[k] = self[k].transpose('h', 'x', ...)
                 setattr(self.__class__, "%s_xylk" % k,
                         property(lambda self, k=k: self[k].isel(x=0).transpose('x', 'h', ...)))
+        elif plane[0] == "xyz":
+            for k in ['WS_eff', 'TI_eff', 'WS', 'WD', 'TI', 'P']:
+                setattr(self.__class__, "%s_xylk" % k, property(lambda self, k=k: self[k].ilk()))
 
     @property
     def XY(self):
@@ -87,7 +98,10 @@ class FlowMap(FlowBox):
             ws = self.WS_xylk
 
         power_xylk = self.windFarmModel.windTurbines.power(ws, **wt_kwargs)
-        return xr.DataArray(power_xylk[:, :, na], self.coords, dims=['y', 'x', 'h', 'wd', 'ws'])
+        if self.plane[0] == "xyz":
+            return power_xylk
+        else:
+            return xr.DataArray(power_xylk[:, :, na], self.coords, dims=['y', 'x', 'h', 'wd', 'ws'])
 
     def aep_xylk(self, normalize_probabilities=False, with_wake_loss=True, **wt_kwargs):
         """Anual Energy Production of a potential wind turbine at all grid positions (x,y)
@@ -147,7 +161,6 @@ class FlowMap(FlowBox):
             if True (default), lines/circles showing the wind turbine rotors are plotted
         ax : pyplot or matplotlib axes object, default None
         """
-        import matplotlib.pyplot as plt
         if cmap is None:
             cmap = 'Blues_r'
         if ax is None:
@@ -172,12 +185,16 @@ class FlowMap(FlowBox):
             x = np.zeros_like(y) + self.plane[1]
             z = self.simulationResult.windFarmModel.site.elevation(x, y)
             ax.plot(y / n, z / n, 'k')
-        else:
+        elif self.plane[0] == 'XY':
+
             # xarray gives strange levels
             # c = data.isel(h=0).plot(levels=levels, cmap=cmap, ax=ax, add_colorbar=plot_colorbar)
             c = ax.contourf(self.X / n, self.Y / n, data.isel(h=0).data, levels=levels, cmap=cmap)
             if plot_colorbar:
                 plt.colorbar(c, label=clabel, ax=ax)
+        else:
+            raise NotImplementedError(
+                f"Plot not supported for FlowMaps based on Points. Use XYGrid, YZGrid or XZGrid instead")
 
         if plot_windturbines:
             self.plot_windturbines(normalize_with=normalize_with, ax=ax)
@@ -187,30 +204,45 @@ class FlowMap(FlowBox):
     def plot_windturbines(self, normalize_with=1, ax=None):
         fm = self.windFarmModel
 
-        def get(k):
-            v = self.simulationResult[k]
-            if 'wd' in v.dims:
-                v = v.sel(wd=self.wd[0])
-            if 'ws' in v.dims:
-                v = v.mean(['ws'])
-            return v.data
-
-        yaw, tilt = get('yaw'), get('tilt')
-
         x_i, y_i = self.simulationResult.x.values, self.simulationResult.y.values
         type_i = self.simulationResult.type.data
         if self.plane[0] in ['XZ', "YZ"]:
             h_i = self.simulationResult.h.values
-            z_i = self.simulationResult.windFarmModel.site.elevation(x_i, y_i)
-            if self.plane[0] == 'XZ':
-                fm.windTurbines.plot_yz(x_i, z_i, h_i, types=type_i, wd=self.wd - 90, yaw=yaw, tilt=tilt,
-                                        normalize_with=normalize_with, ax=ax)
-            else:
-                fm.windTurbines.plot_yz(y_i, z_i, h_i, types=type_i, wd=self.wd, yaw=yaw, tilt=tilt,
-                                        normalize_with=normalize_with, ax=ax)
+            x_ilk, y_ilk = self.simulationResult.x.ilk(), self.simulationResult.y.ilk()
+
+            z_ilk = self.simulationResult.windFarmModel.site.elevation(x_ilk, y_ilk)
+            for l in range(x_ilk.shape[1]):
+                for k in range(x_ilk.shape[2]):
+
+                    wd = self.wd.isel(wd=l) - 90
+
+                    def get(n):
+                        shape = len(x_ilk), len(self.simulationResult.wd), len(self.simulationResult.ws)
+                        if n not in self.simulationResult:
+                            return 0
+                        v = self.simulationResult[n].ilk(shape)
+                        return v[:, l, k]
+                    yaw, tilt = get('yaw'), get('tilt')
+                    if self.plane[0] == 'XZ':
+                        fm.windTurbines.plot_yz(x_ilk[:, l, k], z_ilk[:, l, k], h_i, types=type_i, wd=wd, yaw=yaw, tilt=tilt,
+                                                normalize_with=normalize_with, ax=ax)
+                    else:
+                        fm.windTurbines.plot_yz(y_i, z_ilk[:, l, k], h_i, types=type_i, wd=self.wd, yaw=yaw, tilt=tilt,
+                                                normalize_with=normalize_with, ax=ax)
         else:  # self.plane[0] == "XY":
+            def get(k):
+                if k not in self.simulationResult:
+                    return 0
+                v = self.simulationResult[k]
+                if 'wd' in v.dims:
+                    v = v.sel(wd=self.wd[0])
+                if 'ws' in v.dims:
+                    v = v.mean(['ws'])
+                return v.data
+
+            yaw, tilt = get('yaw'), get('tilt')
             fm.windTurbines.plot_xy(x_i, y_i, type_i,
-                                    wd=self.wd, yaw=yaw, tilt=tilt, normalize_with=normalize_with, ax=ax)
+                                    wd=self.wd.values, yaw=yaw, tilt=tilt, normalize_with=normalize_with, ax=ax)
 
     def plot_wake_map(self, levels=100, cmap=None, plot_colorbar=True, plot_windturbines=True,
                       normalize_with=1, ax=None):
@@ -231,7 +263,10 @@ class FlowMap(FlowBox):
             if True (default), lines/circles showing the wind turbine rotors are plotted
         ax : pyplot or matplotlib axes object, default None
         """
-        return self.plot((self.WS_eff * self.P / self.P.sum(['wd', 'ws'])).sum(['wd', 'ws']), clabel='wind speed [m/s]',
+        sum_dims = [d for d in ['wd', 'time', 'ws'] if d in self.P.dims]
+        WS_eff = (self.WS_eff * self.P / self.P.sum(sum_dims)).sum(sum_dims)
+
+        return self.plot(WS_eff, clabel='wind speed [m/s]',
                          levels=levels, cmap=cmap, plot_colorbar=plot_colorbar,
                          plot_windturbines=plot_windturbines, normalize_with=normalize_with, ax=ax)
 
@@ -299,8 +334,8 @@ class FlowMap(FlowBox):
 
         from py_wake.utils.model_utils import get_model_input
         kwargs = get_model_input(self.windFarmModel, X.flatten(), Y.flatten(), ws=self.ws, wd=self.wd,
-                                 yaw=self.simulationResult.yaw.ilk())
-        dw, hcw, dh = self.windFarmModel.deflectionModel.calc_deflection(**kwargs)
+                                 yaw=self.simulationResult.yaw.ilk(), tilt=self.simulationResult.tilt.ilk())
+        hcw = self.windFarmModel.deflectionModel.calc_deflection(**kwargs)[1]
         Yp = -hcw[0, :, 0, 0].reshape(X.shape)
         ax = ax or plt.gca()
         X, Y, Yp = [v / normalize_with for v in [X, Y, Yp]]
@@ -401,8 +436,8 @@ class YZGrid(Grid):
     def __call__(self, x_i, y_i, h_i, d_i):
         # setup horizontal X,Y grid
         def f(x, N=self.resolution, ext=self.extend):
-            ext *= max(1000, (max(x) - min(x)))
-            return np.linspace(min(x) - ext, max(x) + ext, N)
+            ext *= max(1000, (np.max(x) - np.min(x)))
+            return np.linspace(np.min(x) - ext, np.max(x) + ext, N)
         x, y, z = self.x, self.y, self.z
         if y is None:
             y = f(y_i)
@@ -444,8 +479,8 @@ class XZGrid(YZGrid):
     def __call__(self, x_i, y_i, h_i, d_i):
         # setup horizontal X,Y grid
         def f(x, N=self.resolution, ext=self.extend):
-            ext *= max(1000, (max(x) - min(x)))
-            return np.linspace(min(x) - ext, max(x) + ext, N)
+            ext *= max(1000, (np.max(x) - np.min(x)))
+            return np.linspace(np.min(x) - ext, np.max(x) + ext, N)
         x, y, z = self.x, self.y, self.z
         if x is None:
             x = f(x_i)
@@ -465,6 +500,7 @@ class Points(Grid):
         self.x = x
         self.y = y
         self.h = h
+        self.plane = 'xyz', None
 
     def __call__(self, **_):
         return None, None, self.x, self.y, self.h
