@@ -1,4 +1,4 @@
-import numpy as np
+from py_wake import np
 import pandas as pd
 import matplotlib.pyplot as plt
 from py_wake.examples.data.iea34_130rwt._iea34_130rwt import IEA34_130_1WT_Surrogate, IEA34_130_2WT_Surrogate
@@ -137,7 +137,6 @@ def test_two_turbine_case0_time_series(iea34_130_2WT_Surrogate):
     site = UniformSite(p_wd=[1], ti=ti, ws=ws)
     wfm = NOJ(site, wt, turbulenceModel=STF2017TurbulenceModel())
     sim_res = wfm([0, 0], [0, dist * 130], wd=wdir, time=True, Alpha=shear)
-    assert sim_res.dw_ijl.dims == ('wt', 'wt', 'time')
 
     npt.assert_allclose(ws, ws_ref, rtol=.006)
     # npt.assert_allclose(ti, ws_std_ref / ws_ref, atol=.19)
@@ -162,7 +161,7 @@ def test_functionSurrogate():
     load_sensors = ['del_blade_flap', 'del_blade_edge']
 
     loadFunction = FunctionSurrogates(
-        [TensorflowSurrogate(surrogate_path / s, 'operating') for s in load_sensors],
+        [TensorflowSurrogate.from_dtu_json(surrogate_path / s, 'operating') for s in load_sensors],
         input_parser=lambda ws, TI_eff=.1, Alpha=0: [ws, TI_eff, Alpha])
 
     assert loadFunction.output_keys == [
