@@ -1,13 +1,14 @@
 from py_wake.utils.tensorflow_surrogate_utils import TensorflowSurrogate
 from py_wake.examples.data import example_data_path
-import numpy as np
+from py_wake import np
 from numpy import newaxis as na
 import pytest
 from py_wake.tests import npt
 
 
 def test_TensorflowSurrogate():
-    surrogate = TensorflowSurrogate(example_data_path + "iea34_130rwt/one_turbine/electrical_power", 'operating')
+    surrogate = TensorflowSurrogate.from_dtu_json(example_data_path +
+                                                  "iea34_130rwt/one_turbine/electrical_power", 'operating')
 
     assert surrogate.input_channel_names == ['ws', 'ti', 'shear']
     assert surrogate.output_channel_name == "generator_servo inpvec   2  2: pelec [w]"
@@ -19,7 +20,8 @@ def test_TensorflowSurrogate():
 
 
 def test_bounds_warning():
-    surrogate = TensorflowSurrogate(example_data_path + "iea34_130rwt/one_turbine/electrical_power", 'operating')
+    surrogate = TensorflowSurrogate.from_dtu_json(example_data_path +
+                                                  "iea34_130rwt/one_turbine/electrical_power", 'operating')
     import warnings
     warnings.filterwarnings('error')
     with pytest.raises(UserWarning, match='Input, ws, with value, 3.0 outside range 4.0056388753-24.9807585868'):
