@@ -1,4 +1,4 @@
-import numpy as np
+from py_wake import np
 from py_wake.site.shear import PowerShear
 from py_wake.site.xrsite import XRSite, GlobalWindAtlasSite
 import xarray as xr
@@ -11,15 +11,15 @@ from py_wake.examples.data.hornsrev1 import Hornsrev1Site, V80
 from py_wake.examples.data.iea37._iea37 import IEA37_WindTurbines, IEA37Site
 from py_wake.wind_turbines import WindTurbines
 from py_wake.deficit_models.gaussian import BastankhahGaussian, BastankhahGaussianDeficit
-from py_wake.wind_farm_models.engineering_models import PropagateDownwind, All2AllIterative
+from py_wake.wind_farm_models.engineering_models import PropagateDownwind
 from py_wake.superposition_models import LinearSum
 from py_wake.tests.check_speed import timeit
 from py_wake.site._site import LocalWind
 from py_wake.utils import weibull
-from py_wake.deficit_models.noj import NOJ, NOJDeficit
-from py_wake.flow_map import XYGrid, Points
+from py_wake.deficit_models.noj import NOJ
+from py_wake.flow_map import XYGrid
 import warnings
-from urllib.error import HTTPError
+from urllib.error import HTTPError, URLError
 from py_wake.examples.data.ParqueFicticio._parque_ficticio import ParqueFicticioSite
 from py_wake.utils.gradients import fd, autograd, cs
 
@@ -113,23 +113,23 @@ def test_uniform_local_wind(uniform_site):
 
     wdir_lst = np.arange(0, 360, 90)
     wsp_lst = np.arange(3, 6)
-    lw = site.local_wind(x_i=x_i, y_i=y_i, h_i=100, wd=wdir_lst, ws=wsp_lst)
+    lw = site.local_wind(x=x_i, y=y_i, h=100, wd=wdir_lst, ws=wsp_lst)
     npt.assert_array_equal(lw.WS, 10)
     npt.assert_array_equal(lw.WD, wdir_lst)
     npt.assert_array_equal(lw.TI, 0.1)
     npt.assert_array_equal(lw.P, np.array([0.035972, 0.070002, 0.086432, 0.147379]) * 3)
     npt.assert_array_equal(site.elevation(x_i, y_i), 0)
 
-    lw = site.local_wind(x_i=x_i, y_i=y_i, h_i=100)
+    lw = site.local_wind(x=x_i, y=y_i, h=100)
     npt.assert_array_equal(lw.WD_ilk.shape, (1, 360, 1))
 
     z = np.arange(1, 200)
     zero = [0] * len(z)
 
-    ws100_2 = site.local_wind(x_i=zero, y_i=zero, h_i=z, wd=[0], ws=[10]).WS_ilk[:, 0, 0]
+    ws100_2 = site.local_wind(x=zero, y=zero, h=z, wd=[0], ws=[10]).WS_ilk[:, 0, 0]
 
     site.shear = PowerShear(70, alpha=.3)
-    ws70_3 = site.local_wind(x_i=zero, y_i=zero, h_i=z, wd=[0], ws=[10]).WS_ilk[:, 0, 0]
+    ws70_3 = site.local_wind(x=zero, y=zero, h=z, wd=[0], ws=[10]).WS_ilk[:, 0, 0]
     if 0:
         plt.plot(ws100_2, z)
         plt.plot(ws70_3, z)
@@ -144,7 +144,7 @@ def test_uniform_weibull_local_wind(uniform_weibull_site):
 
     wdir_lst = np.arange(0, 360, 90)
     wsp_lst = np.arange(3, 6)
-    lw = site.local_wind(x_i=x_i, y_i=y_i, h_i=100, wd=wdir_lst, ws=wsp_lst)
+    lw = site.local_wind(x=x_i, y=y_i, h=100, wd=wdir_lst, ws=wsp_lst)
 
     npt.assert_array_equal(lw.WS, [3, 4, 5])
     npt.assert_array_equal(lw.WD, wdir_lst)
@@ -167,7 +167,7 @@ def test_complex_fixed_pos_local_wind(complex_fixed_pos_site):
 
     wdir_lst = np.arange(0, 360, 90)
     wsp_lst = np.arange(3, 6)
-    lw = site.local_wind(x_i=x_i, y_i=y_i, h_i=100, wd=wdir_lst, ws=wsp_lst)
+    lw = site.local_wind(x=x_i, y=y_i, h=100, wd=wdir_lst, ws=wsp_lst)
 
     npt.assert_array_equal(lw.WS, [3, 4, 5] * np.arange(.8, 1.3, .1)[:, na])
     npt.assert_array_equal(lw.WD, (wdir_lst + np.arange(-2, 3)[:, na]) % 360)
@@ -197,7 +197,7 @@ def test_complex_grid_local_wind(complex_grid_site):
 
     wdir_lst = np.arange(0, 360, 90)
     wsp_lst = np.arange(3, 6)
-    lw = site.local_wind(x_i=x_i, y_i=y_i, h_i=100, wd=wdir_lst, ws=wsp_lst)
+    lw = site.local_wind(x=x, y=y, h=100, wd=wdir_lst, ws=wsp_lst)
     if 0:
         c = plt.contourf(X, Y, lw.WS.sel(ws=5).data.reshape(X.shape))
         plt.colorbar(c)
@@ -206,7 +206,7 @@ def test_complex_grid_local_wind(complex_grid_site):
         plt.colorbar(c)
         plt.show()
 
-    lw = site.local_wind(x_i=[2.5, 7.5], y_i=[2.5, 2.5], h_i=100, wd=wdir_lst, ws=wsp_lst)
+    lw = site.local_wind(x=[2.5, 7.5], y=[2.5, 2.5], h=100, wd=wdir_lst, ws=wsp_lst)
     npt.assert_array_almost_equal(lw.WS, [3, 4, 5] * np.array([0.95, 1.15])[:, na])
     npt.assert_array_equal(lw.WD, (wdir_lst + np.array([-.5, 1.5])[:, na]) % 360)
     npt.assert_array_equal(lw.TI, 0.1)
@@ -217,6 +217,17 @@ def test_complex_grid_local_wind(complex_grid_site):
                                          [0.01079997, 0.01656828, 0.02257487]])
 
 
+def test_complex_grid_local_wind_time(complex_grid_site):
+    site = complex_grid_site
+    site.ds['TI'] = (['x', 'y', 'wd'], np.arange(3 * 2 * 13).reshape(3, 2, 13))
+    wdir_lst = [0, 1, 2, 1, 0]
+    wsp_lst = [9, 10, 9, 10, 11]
+    lw = site.local_wind(x=[2.5, 7.5], y=[2.5, 2.5], h=100, wd=wdir_lst, ws=wsp_lst, time=True)
+    assert lw.WS_ilk.shape == (2, 5, 1)
+    assert lw.WD_ilk.shape == (2, 5, 1)
+    assert lw.TI_ilk.shape == (2, 5, 1)
+
+
 @pytest.mark.parametrize('k', ['WD', 'Turning'])
 def test_turning_mean(complex_grid_site, k):
 
@@ -253,7 +264,7 @@ def test_GlobalWindAtlasSite():
 
     try:
         site = GlobalWindAtlasSite(lat, long, height=70, roughness=0.001, ti=0.075)
-    except HTTPError:
+    except (HTTPError, URLError):
         pytest.xfail('HTTPError in GlobalWindAtlasSite')
     ref_mean = weibull.mean(ref.ds.Weibull_A, ref.ds.Weibull_k)
     gwa_mean = weibull.mean(site.ds.Weibull_A, site.ds.Weibull_k)
@@ -290,7 +301,7 @@ def test_wrong_height():
 
     wdir_lst = np.arange(0, 360, 90)
     wsp_lst = np.arange(3, 6)
-    lw = site.local_wind(x_i=x_i, y_i=y_i, h_i=100, wd=wdir_lst, ws=wsp_lst)
+    lw = site.local_wind(x=x_i, y=y_i, h=100, wd=wdir_lst, ws=wsp_lst)
 
 
 def test_wd_independent_site():
@@ -500,8 +511,9 @@ def test_interp(h, wd, ws, h_i, wd_l, ws_k):
     lw = LocalWind(x_i=[25, 50], y_i=[225, 250], h_i=h_i, wd=wd_l, ws=ws_k, time=False, wd_bin_size=1)
 
     for n in ['XYHLK', 'XYHL', 'XYHK', 'K', 'L', 'KL', 'XY', 'H', 'XYH', 'XYL', 'XYK', 'I', 'IL', 'IK', 'ILK']:
-        ip1 = site.interp(site.ds[n], lw.coords)
-        ip2 = ds[n].sel_interp_all(lw.coords)
+        ip1 = site.interp(site.ds[n], lw).squeeze()
+        coords = xr.Dataset(coords={k: [v, ('i', v)][k in 'xyh'] for k, v in lw.coords.items()}).coords
+        ip2 = ds[n].sel_interp_all(coords).squeeze()
         npt.assert_array_equal(ip1.shape, ip2.shape)
         if not np.isnan(ip2).sum():
             npt.assert_array_almost_equal(ip1.data, ip2.data)
@@ -522,16 +534,33 @@ def test_interp_special_cases():
     site = XRSite(ds)
     with pytest.raises(ValueError, match=r"Number of points, i\(=10\), in site data variable, TI, must match "):
         lw = LocalWind(x_i=[25, 50], y_i=[225, 250], h_i=110, wd=wd, ws=ws, time=False, wd_bin_size=1)
-        site.interp(site.ds.TI, lw.coords)
+        site.interp(site.ds.TI, lw)
 
     x = y = np.arange(10)
     lw = LocalWind(x_i=x, y_i=y, h_i=110, wd=wd, ws=ws, time=False, wd_bin_size=1)
-    ip1 = site.interp(site.ds.TI, lw.coords)
-    ip2 = ds.TI.sel_interp_all(lw.coords)
+    ip1 = site.interp(site.ds.TI, lw)
+    coords = xr.Dataset(coords={k: [v, ('i', v)][k in 'xyh'] for k, v in lw.coords.items()}).coords
+    ip2 = ds.TI.sel_interp_all(coords)
     npt.assert_array_equal(ip1.shape, ip2.shape)
     npt.assert_array_almost_equal(ip1.data, ip2.data)
 
 
+@pytest.mark.parametrize('method,ref', [('linear', (2.333333, 2.666666)),
+                                        ('nearest', (3, 4)),
+                                        ({'wd': 'nearest', 'x': 'linear'}, (2, 3)),
+                                        ({'wd': 'linear', 'x': 'nearest'}, (3.333333, 3.666666))
+                                        ])
+def test_interp_mixed_methods(method, ref):
+
+    ds = xr.Dataset({'P': (['x', 'wd'], np.arange(6).reshape((2, 3)))},
+                    coords={'x': [0, 600], 'wd': [0, 30, 60]}
+                    )
+    site = XRSite(ds, interp_method=method)
+
+    lw = LocalWind(x_i=[400], y_i=[0], wd=[10, 20], h_i=100, ws=10, time=False, wd_bin_size=1)
+    npt.assert_array_almost_equal(site.interp(site.ds.P, lw).squeeze(), ref)
+
+
 def test_from_pywasp_pwc_point(pywasp_pwc_point):
 
     site = XRSite.from_pywasp_pwc(pywasp_pwc_point)
@@ -630,10 +659,12 @@ def test_gradients():
     site = ParqueFicticioSite()
     x, y = site.initial_position[0]
 
-    data_vars = ['WS', 'WD', 'ws_lower', 'ws_upper', 'Weibull_A', 'Weibull_k', 'Sector_frequency', 'P', 'TI']
+    data_vars = ['WS_ilk', 'WD_ilk', 'ws_lower', 'ws_upper',
+                 'Weibull_A_ilk', 'Weibull_k_ilk', 'Sector_frequency_ilk', 'P_ilk', 'TI_ilk']
     for data_var in data_vars[1:]:
         def t(x):
-            return site.local_wind(x, y, h_i=[100], wd=270, ws=10)[data_var].values
-        ddx_lst = [grad(t)(x) for grad in [fd, cs, autograd]]
+            return site.local_wind(x, y, h=[100], wd=270, ws=10)[data_var]
+        autograd(t, vector_interdependence=False)(x)
+        ddx_lst = [grad(t, vector_interdependence=False)(x) for grad in [fd, cs, autograd]]
         npt.assert_allclose(ddx_lst[0], ddx_lst[1], rtol=1e-4)
         npt.assert_allclose(ddx_lst[1], ddx_lst[2])
