@@ -1,37 +1,54 @@
-import numpy as np
-import xarray as xr
-import pytest
-from py_wake.examples.data.iea37._iea37 import IEA37_WindTurbines, IEA37Site, IEA37WindTurbinesDeprecated
-from py_wake import NOJ, Fuga, examples
-from py_wake.site._site import UniformSite
-from py_wake.tests import npt
-from py_wake.examples.data.hornsrev1 import HornsrevV80, Hornsrev1Site, wt_x, wt_y, V80
-from py_wake.tests.test_files.fuga import LUT_path_2MW_z0_0_03
-from py_wake.flow_map import HorizontalGrid, XYGrid
-from py_wake.wind_farm_models.engineering_models import All2AllIterative, PropagateDownwind
-from py_wake.deficit_models.noj import NOJDeficit
-from py_wake.superposition_models import SquaredSum, WeightedSum
-from py_wake.deficit_models.selfsimilarity import SelfSimilarityDeficit
-from py_wake.turbulence_models.stf import STF2005TurbulenceModel
-from py_wake.deflection_models.jimenez import JimenezWakeDeflection
-from py_wake.deficit_models.gaussian import IEA37SimpleBastankhahGaussian, IEA37SimpleBastankhahGaussianDeficit, BastankhahGaussian
+import os
+import warnings
+
 from numpy import newaxis as na
+import pytest
+
 import matplotlib.pyplot as plt
-from py_wake.utils.gradients import autograd, cs, fd, plot_gradients
+import pandas as pd
+from py_wake import NOJ, examples
+from py_wake import np
 from py_wake.deficit_models.fuga import FugaDeficit
-from py_wake.superposition_models import LinearSum
+from py_wake.deficit_models.gaussian import IEA37SimpleBastankhahGaussianDeficit, BastankhahGaussian
 from py_wake.deficit_models.no_wake import NoWakeDeficit
+from py_wake.deficit_models.noj import NOJDeficit
+from py_wake.deficit_models.selfsimilarity import SelfSimilarityDeficit
+from py_wake.deflection_models.jimenez import JimenezWakeDeflection
+from py_wake.examples.data.hornsrev1 import HornsrevV80, Hornsrev1Site, wt_x, wt_y, V80
+from py_wake.examples.data.iea37._iea37 import IEA37_WindTurbines, IEA37Site
+from py_wake.flow_map import HorizontalGrid
+from py_wake.literature.iea37_case_study1 import IEA37CaseStudy1
+from py_wake.rotor_avg_models.rotor_avg_model import CGIRotorAvg
+from py_wake.site._site import UniformSite
+from py_wake.superposition_models import LinearSum
+from py_wake.superposition_models import SquaredSum, WeightedSum
+from py_wake.tests import npt
+from py_wake.turbulence_models.stf import STF2005TurbulenceModel
+from py_wake.utils.gradients import autograd, cs, fd, plot_gradients
+from py_wake.utils.model_utils import get_models
+from py_wake.utils.profiling import profileit
+from py_wake.wind_farm_models.engineering_models import All2AllIterative, PropagateDownwind
 from py_wake.wind_farm_models.wind_farm_model import WindFarmModel
 from py_wake.wind_turbines import WindTurbines
-from py_wake.wind_turbines.wind_turbines_deprecated import DeprecatedOneTypeWindTurbines
-import pandas as pd
-import os
-from py_wake.rotor_avg_models.rotor_avg_model import CGIRotorAvg
+from py_wake.wind_turbines._wind_turbines import WindTurbine
+from py_wake.wind_turbines.power_ct_functions import PowerCtFunctionList, PowerCtTabular, PowerCtFunctions
+
+import xarray as xr
 
 
 WindFarmModel.verbose = False
 
 
+class OperatableV80(V80):
+    def __init__(self, method='linear'):
+        V80.__init__(self, method=method)
+        self.powerCtFunction = PowerCtFunctionList(
+            key='operating',
+            powerCtFunction_lst=[PowerCtTabular(ws=[0, 100], power=[0, 0], power_unit='w', ct=[0, 0]),  # 0=No power and ct
+                                 self.powerCtFunction],  # 1=Normal operation
+            default_value=1)
+
+
 def test_wake_model():
     site = IEA37Site(16)
     windTurbines = IEA37_WindTurbines()
@@ -92,18 +109,21 @@ def test_wec():
 def test_str():
     site = IEA37Site(16)
     windTurbines = IEA37_WindTurbines()
-    wf_model = All2AllIterative(site, windTurbines,
-                                wake_deficitModel=NOJDeficit(),
-                                superpositionModel=SquaredSum(),
-                                blockage_deficitModel=SelfSimilarityDeficit(),
-                                deflectionModel=JimenezWakeDeflection(),
-                                turbulenceModel=STF2005TurbulenceModel())
-    assert str(wf_model) == "All2AllIterative(EngineeringWindFarmModel, NOJDeficit-wake, SelfSimilarityDeficit-blockage, RotorCenter-rotor-average, SquaredSum-superposition, JimenezWakeDeflection-deflection, STF2005TurbulenceModel-turbulence)"
+    with warnings.catch_warnings():
+        warnings.simplefilter('ignore', DeprecationWarning)
+        wf_model = All2AllIterative(site, windTurbines,
+                                    wake_deficitModel=NOJDeficit(),
+                                    superpositionModel=SquaredSum(),
+                                    blockage_deficitModel=SelfSimilarityDeficit(),
+                                    rotorAvgModel=CGIRotorAvg(4),
+                                    deflectionModel=JimenezWakeDeflection(),
+                                    turbulenceModel=STF2005TurbulenceModel())
+        assert str(wf_model) == "All2AllIterative(EngineeringWindFarmModel, NOJDeficit-wake, SelfSimilarityDeficit-blockage, SquaredSum-superposition, JimenezWakeDeflection-deflection, STF2005TurbulenceModel-turbulence)"
 
 
 @pytest.mark.parametrize('wake_deficitModel,deflectionModel,superpositionModel',
                          [(NOJDeficit(), None, SquaredSum()),
-                          (IEA37SimpleBastankhahGaussianDeficit(), JimenezWakeDeflection(), WeightedSum())])
+                             (IEA37SimpleBastankhahGaussianDeficit(), JimenezWakeDeflection(), WeightedSum())])
 def test_huge_flow_map(wake_deficitModel, deflectionModel, superpositionModel):
     site = IEA37Site(16)
     windTurbines = IEA37_WindTurbines()
@@ -111,7 +131,8 @@ def test_huge_flow_map(wake_deficitModel, deflectionModel, superpositionModel):
                                    superpositionModel=superpositionModel, deflectionModel=deflectionModel,
                                    turbulenceModel=STF2005TurbulenceModel())
     n_wt = 2
-    flow_map = wake_model(*site.initial_position[:n_wt].T, wd=[0]).flow_map(HorizontalGrid(resolution=810))
+    flow_map = wake_model(*site.initial_position[:n_wt].T, wd=[0], tilt=0,
+                          yaw=0).flow_map(HorizontalGrid(resolution=810))
     # check that deficit matrix > 10MB (i.e. it enters the memory saving loop)
     assert (np.prod(flow_map.WS_eff_xylk.shape) * n_wt * 8 / 1024**2) > 10
     assert flow_map.WS_eff_xylk.shape == (810, 810, 1, 1)
@@ -167,7 +188,6 @@ def test_aep_mixed_type():
 
 def test_dAEP_2wt():
     site = Hornsrev1Site()
-    iea37_site = IEA37Site(16)
 
     wsp_cut_in = 4
     wsp_cut_out = 25
@@ -188,18 +208,10 @@ def test_dAEP_2wt():
 
         return power
 
-    def dpower(wsp):
-        return np.where((wsp > wsp_cut_in) & (wsp <= wsp_rated),
-                        3 * power_rated * (wsp - wsp_cut_in)**2 /
-                        (wsp_rated - wsp_cut_in)**3, 0)
+    wt = WindTurbine(name='test', diameter=130, hub_height=110,
+                     powerCtFunction=PowerCtFunctions(power_function=power, power_unit='w', ct_function=ct))
 
-    def dct(wsp):
-        return wsp * 0  # constant ct
-
-    wt = DeprecatedOneTypeWindTurbines(name='test', diameter=130, hub_height=110,
-                                       ct_func=ct, power_func=power, power_unit='w')
-    wt.set_gradient_funcs(dpower, dct)
-    wfm = IEA37SimpleBastankhahGaussian(site, wt)
+    wfm = PropagateDownwind(site, wt, IEA37SimpleBastankhahGaussianDeficit())
 
     # plot 2 wt case
     x, y = np.array([[0, 130 * 4], [0, 0]], dtype=float)
@@ -214,8 +226,12 @@ def test_dAEP_2wt():
     x_ = x_lst[20]
     ax1.set_title("Center line")
     for grad in [fd, cs, autograd]:
-        dAEPdx = wfm.dAEPdn(0, grad)(x_, y, **kwargs)[1]
+        dAEPdx = grad(wfm.aep, argnum=0)(x_, y, **kwargs)[1]
         npt.assert_almost_equal(dAEPdx / 360, 3.976975605364392e-06, (10, 5)[grad == fd])
+        dAEPdxy = wfm.aep_gradients(grad, ['x', 'y'])(x_, y, **kwargs)
+        npt.assert_almost_equal(dAEPdxy[0][1] / 360, 3.976975605364392e-06, (10, 5)[grad == fd])
+        dAEPdxy = wfm.aep_gradients(grad, ['x', 'y'], wd_chunks=2, x=x_, y=y, **kwargs)
+        npt.assert_almost_equal(dAEPdxy[0][1] / 360, 3.976975605364392e-06, (10, 5)[grad == fd])
         plot_gradients(wfm.aep(x_, y, **kwargs), dAEPdx, x_[1], grad.__name__, step=100, ax=ax1)
     y_lst = np.array([0, 1.]) * np.arange(-100, 100, 5)[:, na]
     ax2.plot(y_lst[:, 1], [wfm.aep(x, y_, **kwargs) for y_ in y_lst])
@@ -224,9 +240,11 @@ def test_dAEP_2wt():
     y_ = y_lst[25]
     ax2.set_title("%d m downstream" % x[1])
     for grad in [fd, cs, autograd]:
-        dAEPdy = wfm.dAEPdn(1, grad)(x, y_, **kwargs)[1]
+        dAEPdy = grad(wfm.aep, argnum=1)(x, y_, **kwargs)[1]
         plot_gradients(wfm.aep(x, y_, **kwargs), dAEPdy, y_[1], grad.__name__, step=50, ax=ax2)
         npt.assert_almost_equal(dAEPdy / 360, 3.794435973860448e-05, (10, 5)[grad == fd])
+        dAEPdxy = wfm.aep_gradients(grad, ['x', 'y'])(x, y_, **kwargs)
+        npt.assert_almost_equal(dAEPdxy[1][1] / 360, 3.794435973860448e-05, (10, 5)[grad == fd])
 
     if 0:
         plt.legend()
@@ -234,23 +252,6 @@ def test_dAEP_2wt():
     plt.close('all')
 
 
-def test_dAEPdx():
-    site = Hornsrev1Site()
-    iea37_site = IEA37Site(16)
-
-    wt = IEA37_WindTurbines()
-    # wt.enable_autograd()
-    wfm = IEA37SimpleBastankhahGaussian(site, wt)
-    x, y = iea37_site.initial_position[np.array([0, 2, 5, 8, 14])].T
-
-    dAEPdxy_autograd = wfm.dAEPdxy(gradient_method=autograd)(x, y)
-    dAEPdxy_cs = wfm.dAEPdxy(gradient_method=cs)(x, y)
-    dAEPdxy_fd = wfm.dAEPdxy(gradient_method=fd)(x, y)
-
-    npt.assert_array_almost_equal(dAEPdxy_autograd, dAEPdxy_cs, 15)
-    npt.assert_array_almost_equal(dAEPdxy_autograd, dAEPdxy_fd, 6)
-
-
 @pytest.mark.parametrize('wake_deficitModel,blockage_deficitModel', [(FugaDeficit(), None),
                                                                      (NoWakeDeficit(), SelfSimilarityDeficit()),
                                                                      (FugaDeficit(), SelfSimilarityDeficit())])
@@ -282,7 +283,6 @@ def test_double_wind_farm_model():
 def test_double_wind_farm_model_All2AllIterative():
     """Check that a new wind farm model does not change results of previous"""
     site = IEA37Site(64)
-    x, y = site.initial_position.T
     x, y = wt_x, wt_y
     windTurbines = IEA37_WindTurbines()
     wfm = All2AllIterative(site, windTurbines, wake_deficitModel=IEA37SimpleBastankhahGaussianDeficit())
@@ -292,6 +292,21 @@ def test_double_wind_farm_model_All2AllIterative():
     npt.assert_array_equal(aep, aep_ref)
 
 
+def test_All2AllIterative_initialize_with_PropagateDownwind():
+    site = IEA37Site(64)
+    x, y = site.initial_position.T
+    windTurbines = V80()
+
+    wfm = All2AllIterative(site, windTurbines, wake_deficitModel=NOJDeficit(),
+                           blockage_deficitModel=SelfSimilarityDeficit())
+    res1 = wfm(x, y).WS_eff
+    i1 = wfm.iterations
+    res2 = wfm(x, y, WS_eff=0).WS_eff
+    i2 = wfm.iterations
+    npt.assert_array_almost_equal(res1, res2)
+    assert i1 <= i2
+
+
 def test_huge_farm():
     site = UniformSite([1], ti=0)
     windTurbines = IEA37_WindTurbines()
@@ -384,7 +399,7 @@ def test_time_series_override_TI():
     npt.assert_array_equal(sim_res.WS, ws)
     npt.assert_array_equal(sim_res.WD, wd)
     npt.assert_array_equal(sim_res.time, t)
-    npt.assert_array_equal(sim_res.TI[0], ti)
+    npt.assert_array_equal(sim_res.TI, ti)
 
 
 def test_time_series_aep():
@@ -400,10 +415,23 @@ def test_time_series_aep():
     npt.assert_allclose(sim_res.aep().sum() / sim_res.aep(with_wake_loss=False).sum(), 0.94, atol=1)
 
 
-def test_time_series_operating():
-    from py_wake.wind_turbines.power_ct_functions import PowerCtFunctionList, PowerCtTabular
+def test_time_series_aep_chunks():
+
     d = np.load(os.path.dirname(examples.__file__) + "/data/time_series.npz")
-    wd, ws, ws_std = [d[k][:6 * 24] for k in ['wd', 'ws', 'ws_std']]
+    wd, ws = [d[k][::100] for k in ['wd', 'ws']]
+    wt = V80()
+    site = Hornsrev1Site()
+    x, y = site.initial_position.T
+    wfm = NOJ(site, wt)
+    sim_res_ref, *_ = profileit(wfm.__call__)(x, y, ws=ws, wd=wd, time=True, verbose=False)
+    sim_res, *_ = profileit(wfm.__call__)(x, y, ws=ws, wd=wd, time=True, wd_chunks=4, verbose=False)
+    npt.assert_almost_equal(sim_res.aep().sum(), sim_res_ref.aep().sum())
+    # npt.assert_allclose(mem, mem_ref / 4, rtol=.3) # fails sometimes when all tests are run
+
+
+def test_time_series_operating():
+    d = np.load(os.path.dirname(examples.__file__) + "/data/time_series.npz")
+    wd, ws = [d[k][:6 * 24] for k in ['wd', 'ws']]
     ws += 3
     t = np.arange(6 * 24)
     wt = V80()
@@ -419,7 +447,7 @@ def test_time_series_operating():
     x, y = site.initial_position.T
     operating = (t < 48) | (t > 72)
     sim_res = wfm(x, y, ws=ws, wd=wd, time=t, operating=operating)
-    npt.assert_array_equal(sim_res.operating[0], operating)
+    npt.assert_array_equal(sim_res.operating, operating)
     npt.assert_array_equal(sim_res.Power[:, operating == 0], 0)
     npt.assert_array_equal(sim_res.Power[:, operating != 0] > 0, True)
 
@@ -432,32 +460,27 @@ def test_time_series_operating():
 
 
 def test_time_series_operating_wrong_shape():
-    from py_wake.wind_turbines.power_ct_functions import PowerCtFunctionList, PowerCtTabular
     d = np.load(os.path.dirname(examples.__file__) + "/data/time_series.npz")
-    wd, ws, ws_std = [d[k][:6 * 24] for k in ['wd', 'ws', 'ws_std']]
+    wd, ws = [d[k][:6 * 24] for k in ['wd', 'ws']]
     ws += 3
     t = np.arange(6 * 24)
-    wt = V80()
+    wt = OperatableV80()
     site = Hornsrev1Site()
 
     # replace powerCtFunction
-    wt.powerCtFunction = PowerCtFunctionList(
-        key='operating',
-        powerCtFunction_lst=[PowerCtTabular(ws=[0, 100], power=[0, 0], power_unit='w', ct=[0, 0]),  # 0=No power and ct
-                             wt.powerCtFunction],  # 1=Normal operation
-        default_value=1)
+
     wfm = NOJ(site, wt)
     x, y = site.initial_position.T
     operating = (t < 48) | (t > 72)
-    with pytest.raises(ValueError, match=r"Argument, operating\(shape=\(1, 144\)\), has unsupported shape."):
-        wfm(x, y, ws=ws, wd=wd, time=t, operating=[operating])
+    with pytest.raises(ValueError, match=r"Argument, operating\(shape=\(2, 144\)\), has unsupported shape."):
+        wfm(x, y, ws=ws, wd=wd, time=t, operating=[operating, operating])
 
 
 def test_aep_wind_atlas_method():
     site = Hornsrev1Site()
 
     wt = IEA37_WindTurbines()
-    wfm = IEA37SimpleBastankhahGaussian(site, wt)
+    wfm = PropagateDownwind(site, wt, IEA37SimpleBastankhahGaussianDeficit(), superpositionModel=SquaredSum())
     x, y = [0], [0]
     wd = np.arange(360)
     aep_lps = wfm(x, y, wd=wd, ws=np.arange(3, 27)).aep(linear_power_segments=True)
@@ -471,3 +494,21 @@ def test_aep_wind_atlas_method():
         plt.show()
     npt.assert_almost_equal(aep_lps.sum(), 16.73490444)
     npt.assert_almost_equal(aep.sum(), 16.69320343)
+
+
+def test_compare_wfm():
+    w = IEA37CaseStudy1(16)
+
+    x, y = w.site.initial_position.T
+
+    res = []
+    for cls in get_models(WindFarmModel):
+        wfm = cls(w.site, w.windTurbines, w.wake_deficitModel, w.superpositionModel)
+        res.append(wfm(x, y).aep().sum().item())
+    npt.assert_allclose(res, res[0])
+
+
+def test_check_input():
+    wfm = IEA37CaseStudy1(16, deflectionModel=JimenezWakeDeflection())
+    with pytest.raises(ValueError, match="'tilt' needed by JimenezWakeDeflection is missing"):
+        wfm([0], [0], yaw=0)
