@@ -1,22 +1,20 @@
 import os
-from numpy import newaxis as na
 import pytest
 import matplotlib.pyplot as plt
-import numpy as np
+from py_wake import np
 from py_wake.deficit_models.gaussian import IEA37SimpleBastankhahGaussianDeficit
 from py_wake.deficit_models.noj import NOJDeficit
 from py_wake.examples.data import wtg_path, hornsrev1
 from py_wake.examples.data.hornsrev1 import V80, wt9_x, wt9_y, Hornsrev1Site
-from py_wake.examples.data.iea37 import iea37_reader
 from py_wake.examples.data.iea37._iea37 import IEA37_WindTurbines, IEA37WindTurbines, IEA37Site
 from py_wake.superposition_models import SquaredSum
 from py_wake.tests import npt
 from py_wake.utils import gradients
 from py_wake.utils.gradients import autograd, plot_gradients
 from py_wake.wind_farm_models.engineering_models import PropagateDownwind, All2AllIterative
-from py_wake.wind_turbines import WindTurbines, WindTurbine, OneTypeWindTurbines, wind_turbines_deprecated,\
-    power_ct_functions
+from py_wake.wind_turbines import WindTurbines, WindTurbine, OneTypeWindTurbines
 from py_wake.wind_turbines.power_ct_functions import PowerCtTabular
+import warnings
 
 
 def get_wfms(wt, site=Hornsrev1Site(), wake_model=NOJDeficit(), superpositionModel=SquaredSum()):
@@ -42,16 +40,17 @@ class V80Deprecated(OneTypeWindTurbines):
 
 def test_DeprecatedWindTurbines():
     v80 = V80Deprecated()
+    with warnings.catch_warnings():
+        warnings.simplefilter('ignore', category=DeprecationWarning)
+        v80d = WindTurbines(names=['V80'], diameters=[80], hub_heights=[70],
+                            ct_funcs=v80._ct_funcs, power_funcs=v80._power_funcs, power_unit='w')
 
-    for wts in [v80,
-                WindTurbines(names=['V80'], diameters=[80], hub_heights=[70],
-                             ct_funcs=v80._ct_funcs, power_funcs=v80._power_funcs, power_unit='w')]:
+    for wts in [v80, v80d]:
 
         types0 = [0] * 9
         for wfm in get_wfms(wts):
-            # wfm = NOJ(Hornsrev1Site(), wts)
             npt.assert_array_equal(wts.types(), [0])
-            npt.assert_almost_equal(wfm.aep(wt9_x, wt9_y, type=types0), 81.2066072392765)
+            npt.assert_almost_equal(wfm.aep(wt9_x, wt9_y, type=types0, yaw=0), 81.2066072392765)
 
 
 def test_WindTurbines():
@@ -94,7 +93,9 @@ def test_twotype_windturbines():
                           hornsrev1.power_curve[:, 0], hornsrev1.power_curve[:, 1] * 1.1, 'w',
                           hornsrev1.ct_curve[:, 1]))
 
-    wts = WindTurbines.from_WindTurbines([v80, v88])
+    with warnings.catch_warnings():
+        warnings.filterwarnings('ignore', category=DeprecationWarning)
+        wts = WindTurbines.from_WindTurbines([v80, v88])
 
     types0 = [0] * 9
     types1 = [0, 0, 0, 1, 1, 1, 0, 0, 0]
@@ -251,9 +252,9 @@ def test_method():
 
     ws_pts = [6.99, 7.01]
 
-    dpdu_linear_pts = autograd(wt_linear.power)(np.array(ws_pts))
-    dpdu_pchip_pts = autograd(wt_pchip.power)(np.array(ws_pts))
-    dpdu_spline_pts = autograd(wt_spline.power)(np.array(ws_pts))
+    dpdu_linear_pts = autograd(wt_linear.power, vector_interdependence=False)(np.array(ws_pts))
+    dpdu_pchip_pts = autograd(wt_pchip.power, vector_interdependence=False)(np.array(ws_pts))
+    dpdu_spline_pts = autograd(wt_spline.power, vector_interdependence=False)(np.array(ws_pts))
 
     if 0:
         wt_dp_label_lst = [(wt_linear, dpdu_linear_pts, 'linear'),
@@ -295,3 +296,8 @@ def test_method():
                                                       (wt_spline, 0.8, dpdu_spline_pts, [205555.17794162, 211859.45965873])]:
         assert np.diff(np.diff(wt.power(ws_lst))).max() < diff_grad_max
         npt.assert_array_almost_equal(dpdu_pts, ref_dpdu_pts)
+
+
+def test_windturbine_from_WindTurbine_lst():
+    with pytest.raises(NotImplementedError, match='Use WindTurbines.from_WindTurbine_lst instead'):
+        WindTurbine.from_WindTurbine_lst([V80(), V80()])
