@@ -1,4 +1,4 @@
-import numpy as np
+from py_wake import np
 import xml.etree.ElementTree as ET
 from matplotlib.patches import Ellipse
 import warnings
@@ -22,7 +22,7 @@ Use WindTurbines(names, diameters, hub_heights, power_ct_funcs) instead""", Depr
         except TypeError:
             return super(WindTurbines, cls).__new__(cls)
 
-    def __init__(self, names, diameters, hub_heights, powerCtFunctions, loadFunctions=None):
+    def __init__(self, names, diameters, hub_heights, powerCtFunctions, **windTurbineFunctions):
         """Initialize WindTurbines
 
         Parameters
@@ -41,8 +41,7 @@ Use WindTurbines(names, diameters, hub_heights, power_ct_funcs) instead""", Depr
         self._hub_heights = np.array(hub_heights)
         assert len(names) == len(diameters) == len(hub_heights) == len(powerCtFunctions)
         self.powerCtFunction = PowerCtFunctionList('type', powerCtFunctions)
-#         if loadFunctions:
-#             self.loadfunction =
+        self.__dict__.update(windTurbineFunctions)
 
     @property
     def function_inputs(self):
@@ -147,7 +146,11 @@ Use WindTurbines(names, diameters, hub_heights, power_ct_funcs) instead""", Depr
         """
         import matplotlib.pyplot as plt
         if types is None:
-            types = np.zeros_like(x)
+            types = np.zeros_like(x).astype(int)
+        else:
+            # ensure same length as x
+            types = (np.zeros(len(x)) + types).astype(int)
+
         if ax is None:
             ax = plt.gca()
         markers = np.array(list("213v^<>o48spP*hH+xXDd|_"))
@@ -155,7 +158,7 @@ Use WindTurbines(names, diameters, hub_heights, power_ct_funcs) instead""", Depr
 
         from matplotlib.patches import Circle
         assert len(x) == len(y)
-        types = (np.zeros_like(x) + types).astype(int)  # ensure same length as x
+
         yaw = np.zeros_like(x) + yaw
         tilt = np.zeros_like(x) + tilt
 
@@ -171,6 +174,7 @@ Use WindTurbines(names, diameters, hub_heights, power_ct_funcs) instead""", Depr
                     circle = Ellipse((x_, y_), 2 * r * np.sin(np.deg2rad(tilt_)), 2 * r,
                                      angle=90 - wd_ + yaw_, ec=colors[t], fc="None")
                     ax.add_artist(circle)
+                    ax.plot(x_, y_, '.', color=colors[t])
 
         for t, m, c in zip(np.unique(types), markers, colors):
             # ax.plot(np.asarray(x)[types == t], np.asarray(y)[types == t], '%sk' % m, label=self._names[int(t)])
@@ -218,16 +222,21 @@ Use WindTurbines(names, diameters, hub_heights, power_ct_funcs) instead""", Depr
         tilt = np.zeros_like(y) + tilt
         y, z, h, D = [v / normalize_with for v in [y, z, h, self.diameter(types)]]
         if isinstance(wd, DataArray):
-            wd = wd.item()
+            wd = wd.values
+
         for i, (y_, z_, h_, d, t, yaw_, tilt_) in enumerate(
                 zip(y, z, h, D, types, yaw, tilt)):
-            ty = y_ - np.cos(np.deg2rad(wd)) * d / 20
-            ax.plot([ty, ty], [z_, z_ + h_], 'k')
-            ax.plot([ty, y_], [z_ + h_, z_ + h_], 'k')
+            if len(np.atleast_1d(wd)) == 1:
+                wd = np.atleast_1d(wd)[0]
+                ty = y_ - np.cos(np.deg2rad(wd)) * d / 20
+                ax.plot([ty, ty], [z_, z_ + h_], 'k')  # tower (d/20 behind rotor)
+                ax.plot([ty, y_], [z_ + h_, z_ + h_], 'k')  # shaft
 
-            circle = Ellipse((y_, h_ + z_), d * np.sin(np.deg2rad(wd - yaw_)),
-                             d, angle=-tilt_, ec=colors[t], fc="None", zorder=32)
-            ax.add_artist(circle)
+                circle = Ellipse((y_, h_ + z_), d * np.sin(np.deg2rad(wd - yaw_)),
+                                 d, angle=-tilt_, ec=colors[t], fc="None", zorder=32)
+                ax.add_artist(circle)
+            else:
+                ax.plot([y_, y_], [h_ + z_ - d / 2, h_ + z_ + d / 2], 'k')  # rotor
             ax.plot(y_, h_, 'None')
 
         for t, m, c in zip(np.unique(types), markers, colors):
@@ -252,8 +261,8 @@ Use WindTurbines(names, diameters, hub_heights, power_ct_funcs) instead""", Depr
         ax2 = ax.twinx()
         ax2.plot(ws, ct, '--')
 
-    @staticmethod
-    def from_WindTurbine_lst(wt_lst):
+    @classmethod
+    def from_WindTurbine_lst(cls, wt_lst):
         """Generate a WindTurbines object from a list of (Onetype)WindTurbines
 
         Parameters
@@ -261,20 +270,21 @@ Use WindTurbines(names, diameters, hub_heights, power_ct_funcs) instead""", Depr
         wt_lst : array_like
             list of (OneType)WindTurbines
         """
-        def get(att):
-            lst = []
-            for wt in wt_lst:
-                lst.extend(getattr(wt, att))
-            return lst
-        return WindTurbines(*[get(n) for n in ['_names', '_diameters', '_hub_heights']] +
-                            [[getattr(wt, 'powerCtFunction') for wt in wt_lst]])
+
+        def key(k):
+            return {'_names': 'names',
+                    '_diameters': 'diameters',
+                    '_hub_heights': 'hub_heights',
+                    'powerCtFunction': 'powerCtFunctions'}.get(k, k)
+
+        attrs = set.intersection(*[set(dir(wt)) - set(dir(WindTurbines)) for wt in wt_lst])
+        return cls(**{key(k): [v for wt in wt_lst for v in np.atleast_1d(getattr(wt, k))] for k in attrs})
 
     @staticmethod
     def from_WindTurbines(wt_lst):
         from py_wake.wind_turbines.wind_turbines_deprecated import DeprecatedWindTurbines
         assert not any([isinstance(wt, DeprecatedWindTurbines) for wt in wt_lst]
                        ), "from_WindTurbines no longer supports DeprecatedWindTurbines"
-        warnings.simplefilter('default', DeprecationWarning)
         warnings.warn("""WindTurbines.from_WindTurbines is deprecated. Use WindTurbines.from_WindTurbine_lst instead""",
                       DeprecationWarning, stacklevel=2)
 
@@ -297,7 +307,7 @@ Use WindTurbines(names, diameters, hub_heights, power_ct_funcs) instead""", Depr
         Note: it is assumed that the power_unit inside multiple .wtg files is the same, i.e., power_unit.
         """
         if isinstance(wtg_file, (list, tuple)):
-            return WindTurbine.from_WindTurbine_lst([WindTurbines.from_WAsP_wtg(f) for f in wtg_file])
+            return WindTurbines.from_WindTurbine_lst([WindTurbines.from_WAsP_wtg(f) for f in wtg_file])
 
         cut_ins = []
         cut_outs = []
@@ -372,6 +382,10 @@ class WindTurbine(WindTurbines):
         for k, v in windTurbineFunctions.items():
             setattr(self, k, v)
 
+    @classmethod
+    def from_WindTurbine_lst(cls, wt_lst):
+        raise NotImplementedError("Use WindTurbines.from_WindTurbine_lst instead")
+
 
 def main():
     if __name__ == '__main__':
