@@ -1,4 +1,4 @@
-import numpy as np
+from py_wake import np
 import inspect
 from abc import ABC, abstractmethod
 import types
@@ -98,8 +98,6 @@ class WindTurbineFunctionList(WindTurbineFunction):
         self.key = key
 
     def _subset(self, arr, mask):
-        if arr is None or isinstance(arr, types.FunctionType):
-            return arr
         return np.broadcast_to(arr.reshape(arr.shape + (1,) * (len(mask.shape) - len(arr.shape))), mask.shape)[mask]
 
     def __call__(self, ws, run_only=slice(None), **kwargs):
@@ -122,18 +120,28 @@ class WindTurbineFunctionList(WindTurbineFunction):
         else:
             if isinstance(run_only, int):
                 o = 0
-                res = np.empty((1,) + np.asarray(ws).shape)
+                res = np.empty(np.asarray(ws).shape)
             else:
                 res = np.empty((len(self.output_keys),) + np.asarray(ws).shape)
                 o = run_only
 
             unique_idx = np.unique(idx)
             idx = np.zeros(ws.shape, dtype=int) + idx.reshape(idx.shape + (1,) * (len(ws.shape) - len(idx.shape)))
-            for i in unique_idx:
-                m = (idx == i)
-                res[o, m] = self.windTurbineFunction_lst[i](
-                    ws[m], run_only=run_only, **{k: self._subset(v, m) for k, v in get_kwargs(i).items()})
-            res = res[o]
+
+            if isinstance(o, int):
+                for i in unique_idx:
+                    m = (idx == i)
+                    r = self.windTurbineFunction_lst[i](ws, run_only=run_only,
+                                                        **{k: v for k, v in get_kwargs(i).items()})
+                    res = np.where(m, r, res)
+                return res
+            else:
+                for i in unique_idx:
+                    m = (idx == i)
+                    res[o, m] = self.windTurbineFunction_lst[i](
+                        ws[m], run_only=run_only, **{k: self._subset(v, m) for k, v in get_kwargs(i).items()})
+                return res[o]
+
         return res
 
 
@@ -160,10 +168,10 @@ class FunctionSurrogates(WindTurbineFunction, ABC):
             return [fs.predict_output(x).reshape(ws.shape) for fs in np.asarray(self.function_surrogate_lst)[run_only]]
 
 #     Commented out as no tests or examples currently uses this class directly
-#     @property
-#     def output_keys(self):
-#         return [fs.output_channel_name for fs in self.function_surrogate_lst]
-#
-#     @property
-#     def wohler_exponents(self):
-#         return [fs.wohler_exponent for fs in self.function_surrogate_lst]
+    # @property
+    # def output_keys(self):
+    #     return [fs.output_channel_name for fs in self.function_surrogate_lst]
+
+    @property
+    def wohler_exponents(self):
+        return [fs.wohler_exponent for fs in self.function_surrogate_lst]
